package org.apache.logging.log4j.core.async;

import com.lmax.disruptor.EventFactory;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.ThreadContext;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.Property;
import org.apache.logging.log4j.core.impl.Log4jLogEvent;
import org.apache.logging.log4j.core.impl.ThrowableProxy;
import org.apache.logging.log4j.core.lookup.StrSubstitutor;
import org.apache.logging.log4j.message.Message;
import org.apache.logging.log4j.message.SimpleMessage;

public class RingBufferLogEvent implements LogEvent {
   private static final long serialVersionUID = 8462119088943934758L;
   public static final RingBufferLogEvent.Factory FACTORY = new RingBufferLogEvent.Factory();
   private transient AsyncLogger asyncLogger;
   private String loggerName;
   private Marker marker;
   private String fqcn;
   private Level level;
   private Message message;
   private transient Throwable thrown;
   private ThrowableProxy thrownProxy;
   private Map<String, String> contextMap;
   private ThreadContext.ContextStack contextStack;
   private String threadName;
   private StackTraceElement location;
   private long currentTimeMillis;
   private boolean endOfBatch;
   private boolean includeLocation;

   public void setValues(AsyncLogger asyncLogger, String loggerName, Marker marker, String fqcn, Level level, Message data, Throwable throwable, Map<String, String> map, ThreadContext.ContextStack contextStack, String threadName, StackTraceElement location, long currentTimeMillis) {
      this.asyncLogger = asyncLogger;
      this.loggerName = loggerName;
      this.marker = marker;
      this.fqcn = fqcn;
      this.level = level;
      this.message = data;
      this.thrown = throwable;
      this.thrownProxy = null;
      this.contextMap = map;
      this.contextStack = contextStack;
      this.threadName = threadName;
      this.location = location;
      this.currentTimeMillis = currentTimeMillis;
   }

   public void execute(boolean endOfBatch) {
      this.endOfBatch = endOfBatch;
      this.asyncLogger.actualAsyncLog(this);
   }

   public boolean isEndOfBatch() {
      return this.endOfBatch;
   }

   public void setEndOfBatch(boolean endOfBatch) {
      this.endOfBatch = endOfBatch;
   }

   public boolean isIncludeLocation() {
      return this.includeLocation;
   }

   public void setIncludeLocation(boolean includeLocation) {
      this.includeLocation = includeLocation;
   }

   public String getLoggerName() {
      return this.loggerName;
   }

   public Marker getMarker() {
      return this.marker;
   }

   public String getLoggerFqcn() {
      return this.fqcn;
   }

   public Level getLevel() {
      if (this.level == null) {
         this.level = Level.OFF;
      }

      return this.level;
   }

   public Message getMessage() {
      if (this.message == null) {
         this.message = new SimpleMessage("");
      }

      return this.message;
   }

   public Throwable getThrown() {
      if (this.thrown == null && this.thrownProxy != null) {
         this.thrown = this.thrownProxy.getThrowable();
      }

      return this.thrown;
   }

   public ThrowableProxy getThrownProxy() {
      if (this.thrownProxy == null && this.thrown != null) {
         this.thrownProxy = new ThrowableProxy(this.thrown);
      }

      return this.thrownProxy;
   }

   public Map<String, String> getContextMap() {
      return this.contextMap;
   }

   public ThreadContext.ContextStack getContextStack() {
      return this.contextStack;
   }

   public String getThreadName() {
      return this.threadName;
   }

   public StackTraceElement getSource() {
      return this.location;
   }

   public long getTimeMillis() {
      return this.currentTimeMillis;
   }

   public void mergePropertiesIntoContextMap(Map<Property, Boolean> properties, StrSubstitutor strSubstitutor) {
      if (properties != null) {
         Map<String, String> map = this.contextMap == null ? new HashMap() : new HashMap(this.contextMap);
         Iterator i$ = properties.entrySet().iterator();

         while(i$.hasNext()) {
            Entry<Property, Boolean> entry = (Entry)i$.next();
            Property prop = (Property)entry.getKey();
            if (!map.containsKey(prop.getName())) {
               String value = (Boolean)entry.getValue() ? strSubstitutor.replace(prop.getValue()) : prop.getValue();
               map.put(prop.getName(), value);
            }
         }

         this.contextMap = map;
      }
   }

   public void clear() {
      this.setValues((AsyncLogger)null, (String)null, (Marker)null, (String)null, (Level)null, (Message)null, (Throwable)null, (Map)null, (ThreadContext.ContextStack)null, (String)null, (StackTraceElement)null, 0L);
   }

   private void writeObject(ObjectOutputStream out) throws IOException {
      this.getThrownProxy();
      out.defaultWriteObject();
   }

   public LogEvent createMemento() {
      return new Log4jLogEvent(this.loggerName, this.marker, this.fqcn, this.level, this.message, this.thrown, this.contextMap, this.contextStack, this.threadName, this.location, this.currentTimeMillis);
   }

   private static class Factory implements EventFactory<RingBufferLogEvent> {
      private Factory() {
      }

      public RingBufferLogEvent newInstance() {
         return new RingBufferLogEvent();
      }

      // $FF: synthetic method
      Factory(Object x0) {
         this();
      }
   }
}
