package org.apache.logging.log4j.spi;

import java.net.URL;
import java.util.Properties;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.status.StatusLogger;

public class Provider {
   private static final Integer DEFAULT_PRIORITY = -1;
   public static final String FACTORY_PRIORITY = "FactoryPriority";
   public static final String THREAD_CONTEXT_MAP = "ThreadContextMap";
   public static final String LOGGER_CONTEXT_FACTORY = "LoggerContextFactory";
   private static final Logger LOGGER = StatusLogger.getLogger();
   private final Integer priority;
   private final String className;
   private final String threadContextMap;
   private final URL url;
   private final ClassLoader classLoader;

   public Provider(Properties props, URL url, ClassLoader classLoader) {
      this.url = url;
      this.classLoader = classLoader;
      String weight = props.getProperty("FactoryPriority");
      this.priority = weight == null ? DEFAULT_PRIORITY : Integer.valueOf(weight);
      this.className = props.getProperty("LoggerContextFactory");
      this.threadContextMap = props.getProperty("ThreadContextMap");
   }

   public Integer getPriority() {
      return this.priority;
   }

   public String getClassName() {
      return this.className;
   }

   public Class<? extends LoggerContextFactory> loadLoggerContextFactory() {
      if (this.className == null) {
         return null;
      } else {
         try {
            Class<?> clazz = this.classLoader.loadClass(this.className);
            if (LoggerContextFactory.class.isAssignableFrom(clazz)) {
               return clazz;
            }
         } catch (Exception var2) {
            LOGGER.error("Unable to create class {} specified in {}", this.className, this.url.toString(), var2);
         }

         return null;
      }
   }

   public String getThreadContextMap() {
      return this.threadContextMap;
   }

   public Class<? extends ThreadContextMap> loadThreadContextMap() {
      if (this.threadContextMap == null) {
         return null;
      } else {
         try {
            Class<?> clazz = this.classLoader.loadClass(this.threadContextMap);
            if (ThreadContextMap.class.isAssignableFrom(clazz)) {
               return clazz;
            }
         } catch (Exception var2) {
            LOGGER.error("Unable to create class {} specified in {}", this.threadContextMap, this.url.toString(), var2);
         }

         return null;
      }
   }

   public URL getUrl() {
      return this.url;
   }
}
